@@ -605,7 +605,9 @@ load_plugin (VinagrePluginsEngine *engine,
 		info->available = FALSE;
 		return FALSE;
 	}
-	
+
+	g_object_set_data (G_OBJECT (info->plugin), "info", info);
+
 	return TRUE;
 }
 
@@ -632,7 +634,6 @@ vinagre_plugins_engine_activate_plugin_real (VinagrePluginsEngine *engine,
 	  return;
 	}
 
-      vinagre_plugin_activate (info->plugin, NULL);
       g_hash_table_insert (engine->priv->protocols, (gpointer)protocol, info->plugin);
       return;
     }
