@@ -38,6 +38,8 @@
 #include "vinagre-ui.h"
 #include "vinagre-window-private.h"
 #include "vinagre-bookmarks-entry.h"
+#include "vinagre-plugin.h"
+#include "vinagre-plugins-engine.h"
 
 #ifdef VINAGRE_ENABLE_AVAHI
 #include "vinagre-mdns.h"
@@ -54,10 +56,11 @@ vinagre_window_dispose (GObject *object)
 {
   VinagreWindow *window = VINAGRE_WINDOW (object);
 
-  if (window->priv->fav_entry_selected)
+  if (!window->priv->dispose_has_run)
     {
-      g_object_unref (window->priv->fav_entry_selected);
-      window->priv->fav_entry_selected = NULL;
+      vinagre_plugins_engine_deactivate_plugins (vinagre_plugins_engine_get_default (),
+						 window);
+      window->priv->dispose_has_run = TRUE;
     }
 
   if (window->priv->manager)
@@ -249,9 +252,10 @@ static void
 activate_recent_cb (GtkRecentChooser *action, VinagreWindow *window)
 {
   VinagreConnection *conn;
-  gchar             *error;
+  gchar             *error, *uri;
 
-  conn = vinagre_connection_new_from_string (gtk_recent_chooser_get_current_uri (action),
+  uri = gtk_recent_chooser_get_current_uri (action);
+  conn = vinagre_connection_new_from_string (uri,
 					     &error,
 					     TRUE);
   if (conn)
@@ -263,6 +267,7 @@ activate_recent_cb (GtkRecentChooser *action, VinagreWindow *window)
     vinagre_utils_show_error (NULL, error ? error : _("Unknown error"), GTK_WINDOW (window));
 
   g_free (error);
+  g_free (uri);
 }
 
 static void
@@ -390,8 +395,9 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
   gtk_ui_manager_add_ui_from_file (manager, vinagre_utils_get_ui_xml_filename (), &error);
   if (error != NULL)
     {
-      g_warning (_("Could not merge vinagre-ui.xml: %s"), error->message);
+      g_critical (_("Could not merge UI XML file: %s"), error->message);
       g_error_free (error);
+      return;
     }
 
   /* Bookmarks */
@@ -427,7 +433,6 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
 		"show-tips", TRUE,
 		"sort-type", GTK_RECENT_SORT_MRU,
 		NULL);
-  gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (window->priv->recent_action), FALSE);
   g_signal_connect (window->priv->recent_action,
 		    "item-activated",
 		    G_CALLBACK (activate_recent_cb),
@@ -475,49 +480,6 @@ fav_panel_size_allocate (GtkWidget     *widget,
 		  NULL);
 }
 
-static void
-fav_panel_activated (VinagreFav            *fav,
-		     VinagreBookmarksEntry *entry,
-		     VinagreWindow         *window)
-{
-  g_return_if_fail (vinagre_bookmarks_entry_get_node (entry) == VINAGRE_BOOKMARKS_ENTRY_NODE_CONN);
-
-  vinagre_cmd_open_bookmark (window, vinagre_bookmarks_entry_get_conn (entry));
-}
-
-static void
-fav_panel_selected (VinagreFav            *fav,
-		    VinagreBookmarksEntry *entry,
-		    VinagreWindow         *window)
-{
-  GtkAction *action1, *action2;
-
-  action1 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "BookmarksDel");
-  action2 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "BookmarksEdit");
-
-  if (window->priv->fav_entry_selected)
-    {
-      g_object_unref (window->priv->fav_entry_selected);
-      window->priv->fav_entry_selected = NULL;
-    }
-
-  if (entry)
-    {
-      window->priv->fav_entry_selected = g_object_ref (entry);
-      gtk_action_set_sensitive (action1, TRUE);
-      gtk_action_set_sensitive (action2, TRUE);
-      /* TODO: Change the menuitem label */
-    }
-  else
-    {
-      /* TODO: Change the menuitem label */
-      gtk_action_set_sensitive (action1, FALSE);
-      gtk_action_set_sensitive (action2, FALSE);
-    }
-}
-
 static void
 vinagre_window_populate_bookmarks (VinagreWindow *window,
 				   const gchar   *group,
@@ -527,10 +489,11 @@ vinagre_window_populate_bookmarks (VinagreWindow *window,
   static guint           i = 0;
   GSList                *l;
   VinagreBookmarksEntry *entry;
-  gchar                 *action_name, *action_label, *path, *tooltip, *icon_name;
+  gchar                 *action_name, *action_label, *path, *tooltip;
   GtkAction             *action;
   VinagreWindowPrivate  *p = window->priv;
   VinagreConnection     *conn;
+  VinagrePlugin         *plugin;
 
   for (l = entries; l; l = l->next)
     {
@@ -568,6 +531,8 @@ vinagre_window_populate_bookmarks (VinagreWindow *window,
 
 	  case VINAGRE_BOOKMARKS_ENTRY_NODE_CONN:
 	    conn = vinagre_bookmarks_entry_get_conn (entry);
+	    plugin = vinagre_plugins_engine_get_plugin_by_protocol (vinagre_plugins_engine_get_default (),
+								    vinagre_connection_get_protocol (conn));
 
 	    action_name = vinagre_connection_get_best_name (conn);
 	    action_label = vinagre_utils_escape_underscores (action_name, -1);
@@ -583,16 +548,17 @@ vinagre_window_populate_bookmarks (VinagreWindow *window,
 				     action_label,
 				     tooltip,
 				     NULL);
-	    icon_name = g_strdup_printf ("application-x-%s", vinagre_connection_get_protocol (conn));
-	    g_object_set (G_OBJECT (action), "icon-name", icon_name, NULL);
-	    g_free (icon_name);
+	    g_object_set (G_OBJECT (action),
+			  "icon-name",
+			  vinagre_plugin_get_icon_name (plugin),
+			  NULL);
 	    g_object_set_data (G_OBJECT (action), "conn", conn);
 	    gtk_action_group_add_action (p->bookmarks_list_action_group,
 					 action);
 	    g_signal_connect (action,
 			     "activate",
-			     G_CALLBACK (vinagre_cmd_bookmarks_open),
-			     window);
+			     G_CALLBACK (vinagre_fav_bookmarks_open),
+			     window->priv->fav_panel);
 
 	    path = g_strdup_printf ("/MenuBar/BookmarksMenu/%s%s", group, parent?parent:"");
 	    gtk_ui_manager_add_ui (p->manager,
@@ -631,8 +597,8 @@ vinagre_window_update_bookmarks_list_menu (VinagreWindow *window)
   for (l = actions; l != NULL; l = l->next)
     {
       g_signal_handlers_disconnect_by_func (GTK_ACTION (l->data),
-					    G_CALLBACK (vinagre_cmd_bookmarks_open),
-					    window);
+					    G_CALLBACK (vinagre_fav_bookmarks_open),
+					    window->priv->fav_panel);
       gtk_action_group_remove_action (p->bookmarks_list_action_group,
 				      GTK_ACTION (l->data));
     }
@@ -668,14 +634,6 @@ create_side_panel (VinagreWindow *window)
 		    "size-allocate",
 		    G_CALLBACK (fav_panel_size_allocate),
 		    window);
-  g_signal_connect (window->priv->fav_panel,
-		    "fav-activated",
-		    G_CALLBACK (fav_panel_activated),
-		    window);
-  g_signal_connect (window->priv->fav_panel,
-		    "fav-selected",
-		    G_CALLBACK (fav_panel_selected),
-		    window);
 }
 
 static void
@@ -687,15 +645,6 @@ init_widgets_visibility (VinagreWindow *window)
   gboolean visible;
   VinagrePrefs *prefs = vinagre_prefs_get_default ();
 
-  /* Remove and Edit bookmarks starts disabled */
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"BookmarksDel");
-  gtk_action_set_sensitive (action, FALSE);
-
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"BookmarksEdit");
-  gtk_action_set_sensitive (action, FALSE);
-
   /* side panel visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewSidePanel");
@@ -770,8 +719,8 @@ vinagre_window_init (VinagreWindow *window)
   gtk_window_set_default_icon_name ("vinagre");
 
   window->priv = VINAGRE_WINDOW_GET_PRIVATE (window);
-  window->priv->fav_entry_selected = NULL;
   window->priv->fullscreen = FALSE;
+  window->priv->dispose_has_run = FALSE;
 
   main_box = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), main_box);
@@ -815,6 +764,10 @@ vinagre_window_init (VinagreWindow *window)
                             G_CALLBACK (vinagre_window_update_bookmarks_list_menu),
                             window);
 #endif
+
+  vinagre_plugins_engine_activate_plugins (vinagre_plugins_engine_get_default (),
+					   window);
+
 }
 
 VinagreNotebook *
@@ -985,21 +938,31 @@ VinagreTab *
 vinagre_window_conn_exists (VinagreWindow *window, VinagreConnection *conn)
 {
   VinagreConnection *c;
-  VinagreTab *tab = NULL;
+  VinagreTab *tab;
+  const gchar *host, *protocol;
+  gint port;
   GList *l;
 
   g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
   g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), NULL);
 
+  host = vinagre_connection_get_host (conn);
+  protocol = vinagre_connection_get_protocol (conn);
+  port = vinagre_connection_get_port (conn);
+
+  if (!host || !protocol)
+    return NULL;
+
   l = vinagre_window_get_connections (window);
+  tab = NULL;
 
   while (l != NULL)
     {
       c = VINAGRE_CONNECTION (l->data);
 
-      if (!strcmp (vinagre_connection_get_host (conn), vinagre_connection_get_host (c)) &&
-	  vinagre_connection_get_port (conn) == vinagre_connection_get_port (c) &&
-	  vinagre_connection_get_protocol (conn) == vinagre_connection_get_protocol (c))
+      if (!strcmp (host, vinagre_connection_get_host (c)) &&
+	  !strcmp (protocol, vinagre_connection_get_protocol (c)) &&
+	  port == vinagre_connection_get_port (c))
 	{
 	  tab = g_object_get_data (G_OBJECT (c), VINAGRE_TAB_KEY);
 	  break;
